==> sigaddons-7.0.21.12.14.tar.gz: sigaddons-7.0.21.12.14/sigaddons/automacao/inherit/estoquepedidos/pedido.py <==
# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils.calcDigito import ean
from openerp.osv.fields import float_round
from decimal import Decimal, ROUND_DOWN, ROUND_UP


class estoquepedidos_pedido_capa(osv.Model):

    _inherit = 'estoquepedidos.pedido.capa'

    def _pedcap_ajusta_qtd(self, qtd, vr_unitario, valor):
        total_item = float(
            Decimal(str(qtd * vr_unitario)).quantize(Decimal('.01'), rounding=ROUND_DOWN))
        if float_round(total_item, 2) == float_round(valor, 2):
            return qtd, vr_unitario

        elif total_item < valor:
            total_item = float(
                Decimal(str(qtd * vr_unitario)).quantize(Decimal('.01'), rounding=ROUND_UP))
            if float_round(total_item, 2) == float_round(valor, 2):
                return qtd, vr_unitario
            return self._pedcap_ajusta_qtd(qtd + 0.001, vr_unitario, valor)

        return False

    def _pedcap_ajusta_valor(self, qtd, vr_unitario, valor):
        total_item = ('%s' % (qtd * vr_unitario))
        total_item = float(total_item[:total_item.index('.') + 3])
        if total_item == valor:
            return qtd, vr_unitario

        elif total_item < valor:
            self.total_item_aprox = total_item
            return self._pedcap_ajusta_valor(qtd, vr_unitario + 0.01, valor)

        return False

    def pedcap_busca_produto_balanca(self, cr, uid, cod, context=None):
        context = context and context.copy() or {}
        digitos = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro',
                                              ['estpar_pedido_separar'], context=context).get('estpar_qtd_digitos') or 5
        cod_prod = cod[1:digitos + 1]
        if digitos:
            cod_prod = '%s%d' % (cod_prod, ean(int(cod_prod)))

        cod_prod = '%014d' % int(cod_prod)
        obj_cb = self.pool.get('automacao.codigo.barra')
        cb_ids = obj_cb.search(cr, uid, [('codigo_barra', '=', cod_prod)], context=context)
        if not cb_ids:
            return {'warning': {'title': _('Atencao'),
                                'message': _('Codigo nao cadastrado na tabela de automacao!')}
                    }
        brw_cb = obj_cb.browse(cr, uid, cb_ids[0], context)
        brw_produto = brw_cb.produto_id
        if not brw_produto.produto_de_balanca:
            return {'warning': {'title': _('Atencao'),
                                'message': _("O item '%s' nao e de balanca") % cod_prod}
                    }
        if not brw_produto.preco_venda:
            return {'warning': {'title': _('Atencao'),
                                'message': _("O item '%s' nao tem preco de venda") % cod_prod}
                    }

        inteiro = cod[1 + digitos:-3]
        casas = cod[-3:-1]
        vlr_str = '%d.%02d' % (int(inteiro), int(casas))
        total_item_aprox = 0
        total_item = float(vlr_str)
        qtd = float_round(total_item / brw_produto.preco_venda, 3)
        qtda_t = self._pedcap_ajusta_qtd(qtd, brw_produto.preco_venda, total_item)
        if not qtda_t:
            qtda_t = self._pedcap_ajusta_valor(qtd, brw_produto.preco_venda + float('0.01'), total_item)
            if not qtda_t:
                qtda_t = qtd, total_item_aprox

        return total_item_aprox or qtda_t[0], qtda_t[1], total_item, brw_cb

    def pedcap_simp_onchange_codigo_barra(self, cr, uid, ids, company_id, codigo_barra, item_ids,
                                          context=None):
        context = context and context.copy() or {}
        res = {}
        if not codigo_barra:
            return res
        value = {'pedcap_simp_codigo_barra': ''}
        quantidade_codigo = codigo_barra.split('*')
        if len(quantidade_codigo) == 1:
            quantidade = 1
            codigo_barra = quantidade_codigo[0]
        else:
            quantidade = float(quantidade_codigo[0]) if quantidade_codigo[0].isdigit() else 1
            codigo_barra = quantidade_codigo[1]
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        local_id = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque', 'local_venda_padrao_id',
                                                                    context=context).get('local_venda_padrao_id')
        if not local_id:
            return {'warning': {'title': _('Atencao'),
                                'message': _('Local para saida nao cadastrado nos parametros do estoque')},
                    'value': value
                    }

        if codigo_barra.startswith('2') and len(codigo_barra) == 13:
            resp = self.pedcap_busca_produto_balanca(cr, uid, codigo_barra, context)
            if resp:
                if resp == dict:
                    resp.update({'value': {'pedcap_simp_codigo_barra': ''}})
                    return resp

                qtd, vr_unitario, valor, brw_cb = resp
                vals_pedido_item = {'pedite_codigo_barra_id': brw_cb.id,
                                    'produto_id': brw_cb.produto_id.id,
                                    'grade_id': brw_cb.grade_id.id,
                                    'qtde_pedida': qtd,
                                    'vr_unitario': vr_unitario,
                                    'pei_preco_venda': vr_unitario,
                                    'valor_total': valor,
                                    'local_id': local_id,
                                    'produto_embalagem_id': brw_cb.produto_embalagem_id.id
                                    }
                item_ids.append([0, False, vals_pedido_item])
                value.update({'pedido_item_ids': item_ids})
                res['value'] = value
                return res

        codigo_barra_ids = obj_automacao_codigo_barra.search(
            cr, uid, [('codigo_barra', '=', codigo_barra.zfill(14))], context=context)
        if not codigo_barra_ids:
            res['value'] = value
            return res

        brw_automacao_codigo_barra = obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids[0], context)
        produto_id = brw_automacao_codigo_barra.produto_id.id
        grade_id = brw_automacao_codigo_barra.grade_id.id

        produto_preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_id, epc_grade_ids=grade_id, epc_company_id=company_id)
        dados_preco_venda = produto_preco_venda.get(produto_id) or {}
        preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get('preco_venda')
        if not preco_venda:
            produto_dados = self.pool.get('estoque.produto').read(cr, uid, produto_id, ['name'],
                                                                  context=context)
            if isinstance(produto_dados, list):
                produto_dados = produto_dados[0]
            produto_name = produto_dados.get('name') or ''
            return {
                'warning': {'title': _('Atencao'),
                            'message': _('O produto: %s, nao possui preco de venda '
                                         'cadastrado!') % produto_name},
                'value': value
            }

        vals_pedido_item = {
            'pedite_codigo_barra_id': brw_automacao_codigo_barra.id,
            'produto_id': produto_id,
            'grade_id': grade_id,
            'qtde_pedida': quantidade,
            'vr_unitario': preco_venda,
            'pei_preco_venda': preco_venda,
            'local_id': local_id,
            'produto_embalagem_id': brw_automacao_codigo_barra.produto_embalagem_id.id
        }
        item_ids.append([0, False, vals_pedido_item])
        value.update({'pedido_item_ids': item_ids})
        res['value'] = value
        return res

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).default_get(cr, uid, fields, context)
        res.update({'pedcap_simp_state': 'B'})
        return res

==> sigaddons-7.0.2310.2.tar.gz: sigaddons-7.0.2310.2/sigaddons/automacao/inherit/estoquepedidos/pedido.py <==
# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils.calcDigito import ean
from openerp.osv.fields import float_round
from decimal import Decimal, ROUND_DOWN, ROUND_UP


class estoquepedidos_pedido_capa(osv.Model):

    _inherit = 'estoquepedidos.pedido.capa'

    def _pedcap_ajusta_qtd(self, qtd, vr_unitario, valor):
        total_item = float(
            Decimal(str(qtd * vr_unitario)).quantize(Decimal('.01'), rounding=ROUND_DOWN))
        if float_round(total_item, 2) == float_round(valor, 2):
            return qtd, vr_unitario

        elif total_item < valor:
            total_item = float(
                Decimal(str(qtd * vr_unitario)).quantize(Decimal('.01'), rounding=ROUND_UP))
            if float_round(total_item, 2) == float_round(valor, 2):
                return qtd, vr_unitario
            return self._pedcap_ajusta_qtd(qtd + 0.001, vr_unitario, valor)

        return False

    def _pedcap_ajusta_valor(self, qtd, vr_unitario, valor):
        total_item = ('%s' % (qtd * vr_unitario))
        total_item = float(total_item[:total_item.index('.') + 3])
        if total_item == valor:
            return qtd, vr_unitario

        elif total_item < valor:
            self.total_item_aprox = total_item
            return self._pedcap_ajusta_valor(qtd, vr_unitario + 0.01, valor)

        return False

    def pedcap_busca_produto_balanca(self, cr, uid, cod, context=None):
        context = context and context.copy() or {}
        digitos = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro',
                                              ['estpar_qtd_digitos'], context=context).get('estpar_qtd_digitos') or 5
        cod_prod = cod[1:digitos + 1]
        if digitos:
            cod_prod = '%s%d' % (cod_prod, ean(int(cod_prod)))

        cod_prod = '%014d' % int(cod_prod)
        obj_cb = self.pool.get('automacao.codigo.barra')
        cb_ids = obj_cb.search(cr, uid, [('codigo_barra', '=', cod_prod)], context=context)
        if not cb_ids:
            return {'warning': {'title': _('Atencao'),
                                'message': _('Codigo nao cadastrado na tabela de automacao!')}
                    }
        brw_cb = obj_cb.browse(cr, uid, cb_ids[0], context)
        brw_produto = brw_cb.produto_id
        if not brw_produto.produto_de_balanca:
            return {'warning': {'title': _('Atencao'),
                                'message': _("O item '%s' nao e de balanca") % cod_prod}
                    }
        if not brw_produto.preco_venda:
            return {'warning': {'title': _('Atencao'),
                                'message': _("O item '%s' nao tem preco de venda") % cod_prod}
                    }

        inteiro = cod[1 + digitos:-3]
        casas = cod[-3:-1]
        vlr_str = '%d.%02d' % (int(inteiro), int(casas))
        total_item_aprox = 0
        total_item = float(vlr_str)
        qtd = float_round(total_item / brw_produto.preco_venda, 3)
        qtda_t = self._pedcap_ajusta_qtd(qtd, brw_produto.preco_venda, total_item)
        if not qtda_t:
            qtda_t = self._pedcap_ajusta_valor(qtd, brw_produto.preco_venda + float('0.01'), total_item)
            if not qtda_t:
                qtda_t = qtd, total_item_aprox

        return total_item_aprox or qtda_t[0], qtda_t[1], total_item, brw_cb

    def pedcap_simp_onchange_codigo_barra(self, cr, uid, ids, company_id, codigo_barra, item_ids,
                                          context=None):
        context = context and context.copy() or {}
        res = {}
        if not codigo_barra:
            return res
        value = {'pedcap_simp_codigo_barra': ''}
        quantidade_codigo = codigo_barra.split('*')
        if len(quantidade_codigo) == 1:
            quantidade = 1
            codigo_barra = quantidade_codigo[0]
        else:
            quantidade = float(quantidade_codigo[0]) if quantidade_codigo[0].isdigit() else 1
            codigo_barra = quantidade_codigo[1]
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        local_id = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque', 'local_venda_padrao_id',
                                                                    context=context).get('local_venda_padrao_id')
        if not local_id:
            return {'warning': {'title': _('Atencao'),
                                'message': _('Local para saida nao cadastrado nos parametros do estoque')},
                    'value': value
                    }

        if codigo_barra.startswith('2') and len(codigo_barra) == 13:
            resp = self.pedcap_busca_produto_balanca(cr, uid, codigo_barra, context)
            if resp:
                if resp == dict:
                    resp.update({'value': {'pedcap_simp_codigo_barra': ''}})
                    return resp

                qtd, vr_unitario, valor, brw_cb = resp
                vals_pedido_item = {'pedite_codigo_barra_id': brw_cb.id,
                                    'produto_id': brw_cb.produto_id.id,
                                    'grade_id': brw_cb.grade_id.id,
                                    'qtde_pedida': qtd,
                                    'vr_unitario': vr_unitario,
                                    'pei_preco_venda': vr_unitario,
                                    'valor_total': valor,
                                    'local_id': local_id,
                                    'produto_embalagem_id': brw_cb.produto_embalagem_id.id
                                    }
                item_ids.append([0, False, vals_pedido_item])
                value.update({'pedido_item_ids': item_ids})
                res['value'] = value
                return res

        codigo_barra_ids = obj_automacao_codigo_barra.search(
            cr, uid, [('codigo_barra', '=', codigo_barra.zfill(14))], context=context)
        if not codigo_barra_ids:
            res['value'] = value
            return res

        brw_automacao_codigo_barra = obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids[0], context)
        produto_id = brw_automacao_codigo_barra.produto_id.id
        grade_id = brw_automacao_codigo_barra.grade_id.id

        produto_preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_id, epc_grade_ids=grade_id, epc_company_id=company_id)
        dados_preco_venda = produto_preco_venda.get(produto_id) or {}
        preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get('preco_venda')
        if not preco_venda:
            produto_dados = self.pool.get('estoque.produto').read(cr, uid, produto_id, ['name'],
                                                                  context=context)
            if isinstance(produto_dados, list):
                produto_dados = produto_dados[0]
            produto_name = produto_dados.get('name') or ''
            return {
                'warning': {'title': _('Atencao'),
                            'message': _('O produto: %s, nao possui preco de venda '
                                         'cadastrado!') % produto_name},
                'value': value
            }

        vals_pedido_item = {
            'pedite_codigo_barra_id': brw_automacao_codigo_barra.id,
            'produto_id': produto_id,
            'grade_id': grade_id,
            'qtde_pedida': quantidade,
            'vr_unitario': preco_venda,
            'pei_preco_venda': preco_venda,
            'local_id': local_id,
            'produto_embalagem_id': brw_automacao_codigo_barra.produto_embalagem_id.id
        }
        item_ids.append([0, False, vals_pedido_item])
        value.update({'pedido_item_ids': item_ids})
        res['value'] = value
        return res

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).default_get(cr, uid, fields, context)
        res.update({'pedcap_simp_state': 'B'})
        return res
